/*******************************************************************************
* File Name: main.c
*
* Version: 1.0
*
* Description:
* This project demonstrates a 16 Bit Low Pass Filter Operation. For this, we are using
* ADC configured for 16 bit resolution and DAC configured for 8 bit resolution
*
* Firmware Dependency:
* PSoC Creator 3.3 SP1 and above
*
* Hardware Dependency:
* CY8CKIT-050 PSoC 5LP Development Kit
*
********************************************************************************
* Copyright 2014-15, Cypress Semiconductor Corporation. All rights reserved.
* This software is owned by Cypress Semiconductor Corporation and is protected
* by and subject to worldwide patent and copyright laws and treaties.
* Therefore, you may use this software only as provided in the license agreement
* accompanying the software package from which you obtained this software.
* CYPRESS AND ITS SUPPLIERS MAKE NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* WITH REGARD TO THIS SOFTWARE, INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT,
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*******************************************************************************/

#include <project.h>

/* Necessary defines for DMA Configuration. Request Per Burst is set to 1 as ADC
End Of Conversion triggers DMA. Two Bytes are transferred per burst to write into 
lower and middle Staging Registers of the filter block */

#define DMA_BYTES_PER_BURST     (2u)
#define DMA_REQUEST_PER_BURST   (1u)
#define DMA_SRC_BASE            (CYDEV_PERIPH_BASE)
#define DMA_DST_BASE            (CYDEV_PERIPH_BASE)
#define TRANSFER_COUNT          (2u)

/* General constat defines */
#define FILTER_DATA_ALIGN       (0x05u)
#define SHIFT_EIGHT             (0x08u)
#define RESCALING_FACTOR        (0x80u)

#define FiltA

/* Function Prototypes */
/* Interrpt service routine to read the filterd data */
CY_ISR_PROTO(Filter_Done);

/* Function to configure DMA Channel */
void DMA_Config(void);


/*******************************************************************************
* Function Name: main
********************************************************************************
*
* Summary:
*  Starts all the componnets and enables global interrupts
*  Output to VDAC once valid data is present in Filter's Holding Register

* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
int main()
{
	/* Enable Global Interrupts */
    CYGlobalIntEnable;  
    
    /* Start all components used on schematic */
    ADC_DelSig_Start();
    Filter_Start();
	VDAC8_Start();
    ADC_DelSig_IRQ_Start();
    isr_Filter_StartEx(Filter_Done);
    
    /* For 9-16 bits filter resolution, coherency should be mid, Dalign should be enabled, 
    filter stage pointer will be Filter_STAGEA_PTR */
    Filter_DALIGN_REG =  Filter_DALIGN_REG | FILTER_DATA_ALIGN;
	Filter_SetCoherency(Filter_CHANNEL_A,Filter_KEY_MID );
	Filter_SetCoherency(Filter_CHANNEL_B,Filter_KEY_MID );    
    
    /* User-implemented function to set-up DMA */
    DMA_Config();
    
    /* Start the ADC Conversion */ 
    ADC_DelSig_StartConvert();
	
    for(;;)
    {
        /* Filtered data will be written to VDAC in the filter_done interrupt */ 
    }
} /* End of main */


/*******************************************************************************
* Interrupt
********************************************************************************
* Interrupt generated on Filter sample-ready. Interrupt handle:Filter_Done
*
* Summary:
*  The interrupt performs following functions:
*   1: Reads the left-justified register for Filter Channel A
*   2: Writes the most significant 8 bits of filterd data to VDAC as VDAC is 8 bit
*
*******************************************************************************/
CY_ISR(Filter_Done)
{
	uint8 vdata;
    uint32 Filter_Result;
	
    /* Read the filter value from filter's holding register */
#ifdef FiltA    
	Filter_Result = Filter_Read24(Filter_CHANNEL_A);
#else
    Filter_Result = Filter_Read24(Filter_CHANNEL_B); 
#endif    
    /* Write the MSB eight bits to the VDAC */
	vdata = Filter_Result >> SHIFT_EIGHT;	
    
    /* Add 128 to convert 8 bit signed number to 8 bit unsigned number */
    VDAC8_SetValue(vdata + RESCALING_FACTOR);
    
	
}

/*******************************************************************************
* Function Name: DMA_Config
********************************************************************************
*
* Summary:
*  Initializes and sets up DMA for use (generated by DMA Wizard)
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void DMA_Config(void)
{  
    /* Variable declarations for DMA */
    uint8 channelHandle;
    uint8 tdChanA;

    /* Configure the DMA to Transfer the data in 1 burst with individual trigger
     * for each burst.*/
    channelHandle = DMA_DmaInitialize(DMA_BYTES_PER_BURST, DMA_REQUEST_PER_BURST, 
                                         HI16(DMA_SRC_BASE), HI16(DMA_DST_BASE));
	
    /* This function allocates a TD for use with an initialized DMA channel */
    tdChanA = CyDmaTdAllocate();
    
    /* Source and Destination address increments are not needed as we are using 2 
    byte transfers and Spoke Width is 16 bit */
	CyDmaTdSetConfiguration(tdChanA, TRANSFER_COUNT, tdChanA, 0u);
    
    /* Set the source address as ADC_DelSig and the destination as Filter Channel A*/
#ifdef FiltA
   	CyDmaTdSetAddress(tdChanA, LO16((uint32)ADC_DelSig_DEC_SAMP_16B_PTR),
                                        LO16((uint32)Filter_STAGEA_PTR));
#else
    CyDmaTdSetAddress(tdChanA, LO16((uint32)ADC_DelSig_DEC_SAMP_16B_PTR),
                                        LO16((uint32)Filter_STAGEB_PTR));    
#endif    
    
    /* Set tdChanA to be the initial TD associated with channelHandle */
	CyDmaChSetInitialTd(channelHandle, tdChanA);
	
    /* Enable the DMA channel represented by channelHandle and preserve the TD */
    CyDmaChEnable(channelHandle, 1u);
}

/* [] END OF FILE */
